@@ -15,16 +15,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#ifndef __CANVAS_H__
-#define __CANVAS_H__
+#ifndef __BUFFER_H__
+#define __BUFFER_H__
 
-#include "tag.h"
-#include "memory.h"
+#include "cow.h"
 
 
 #define GIMP_BUFFER(b)        ((GimpBuffer*)(b))
 #define GIMP_BUFFER_CLASS(b)  ((GimpBufferClass*)(b))
 
+/* need to reconcile Tag with ColorModel */
+typedef guint Tag;
+#define tag_valid(t) TRUE
+#define tag_bytes(t) (t)
+
 
 
 typedef struct _GimpBuffer GimpBuffer;
@@ -36,12 +40,14 @@ typedef struct _GimpPixelArray GimpPixelArray;
 typedef struct _GimpMemStatus GimpMemStatus;
 
 
+
 typedef enum
 {
   BUFFER_NONE,
   BUFFER_FLAT,
   BUFFER_TILED,
-  BUFFER_SHM
+  BUFFER_SHM,
+  BUFFER_PTR
 } BufferType;
 
 
@@ -71,27 +77,56 @@ typedef enum
 } Use;
 
 
+
+/* ----------------------------------------------------------- */
+
 struct _GimpPoint {
   gint x;
   gint y;
 };
 
 
+
+/* ----------------------------------------------------------- */
+
 struct _GimpArea {
   GimpPoint a;
   GimpPoint b;
 };
 
+#define ga_width(ga)  ((ga)->b.x - (ga)->a.x)
+#define ga_height(ga) ((ga)->b.y - (ga)->a.y)
+
+
+/* ----------------------------------------------------------- */
 
 struct _GimpPortion
 {
-  GimpArea focus;
-  GimpArea bounds;
+  /* the original area passed to gimp_buffer_focus */
+  GimpArea orig;
 
-  /* subclass specific data */
-  gint tilenum;
+  /* shrunk to whole tiles */
+  GimpArea min;
+  
+  /* expanded to whole tiles */
+  GimpArea max;
+  
+  /* clipped to single tile */
+  GimpArea focus;
+
+  /* clipped to single tile and expanded to whole tile */
+  GimpArea bounds;
 };
 
+#define gp_orig(gp)    (&(gp)->orig)
+#define gp_min(gp)     (&(gp)->min)
+#define gp_max(gp)     (&(gp)->max)
+#define gp_focus(gp)   (&(gp)->focus)
+#define gp_bounds(gp)  (&(gp)->bounds)
+
+
+
+/* ----------------------------------------------------------- */
 
 struct _GimpPixelArray
 {
@@ -111,6 +146,22 @@ struct _GimpPixelArray
   gint offset;
 };
 
+#define gpa_tag(gpa)         ((gpa)->tag)
+#define gpa_width(gpa)       ((gpa)->width)
+#define gpa_height(gpa)      ((gpa)->height)
+#define gpa_pixstride(gpa)   ((gpa)->pixstride)
+#define gpa_rowstride(gpa)   ((gpa)->rowstride)
+#define gpa_data(gpa)        ((gpa)->data)
+#define gpa_offset(gpa)      ((gpa)->offset)
+
+#define gpa_dataptr(gpa)     ((void*) \
+                              (guchar_d((gpa)->data) + \
+                               (gpa)->offset))
+
+
+
+
+/* ----------------------------------------------------------- */
 
 struct _GimpMemStatus
 {
@@ -125,30 +176,53 @@ struct _GimpMemStatus
 };
 
 
+
+/* ----------------------------------------------------------- */
+
+/* custom validator.  the void ptr is the raw memory located inside
+   area in the buffer */
+typedef void (*GimpValidator) (GimpBuffer *, GimpArea *, void *);
+
+
+/* the member vars have funny names to avoid surprises from the
+   TileManager typedef */
 struct _GimpBuffer
 {
   /* the class object */
   void * klass;
 
-  /* buffer size */
-  gint width;    /* in pixels */
-  gint height;   /* in pixels */
-  gint depth;    /* in bytes  */
-
-  /* the tiling characteristics */
+  /* the xy dimensions */
   struct {
+    
+    /* size of image data in pixels */
+    gint len;
+    
+    /* size of tile */
     gint period;
-    gint phase;
-  } x, y;
 
-  /* pixel layout */
-  Tag tag;
+    /* offset of image data into first tile */
+    gint offset;
 
-  /* should invalid memory be validated when used? */
-  gboolean autovalidate;
+  } _x, _y;    
 
-  /* should unalloced memory be alloced when validated? */
-  gboolean autoalloc;
+  /* the z dimension */
+  struct {
+    
+    /* size in bytes */
+    gint len;
+
+    /* pixel layout. merge with jays ColorModel concept */
+    Tag tag;
+
+  } _z;
+  
+  /* custom validator */
+  GimpValidator vfunc;
+  void * vdata;
+  
+  /* temporary, gimp expects buffers to have a position */
+  gint qq_x;
+  gint qq_y;
 };
 
 
@@ -159,14 +233,15 @@ struct _GimpBufferClass
 
   /* virtual method table */
   void      (* delete)  (GimpBuffer        *buffer);
-  gboolean  (* focus)   (GimpBuffer        *buffer,
-                         GimpPortion       *portion,
-                         GimpArea          *roi);
   gboolean  (* alloc)   (GimpBuffer        *buffer,
-                         GimpPortion       *portion,
+                         GimpArea          *area,
                          Alloc              how);
+  gboolean  (* map)     (GimpBuffer        *dst,
+                         GimpArea          *darea,
+                         GimpBuffer        *src,
+                         GimpArea          *sarea);
   gboolean  (* validate)(GimpBuffer        *buffer,
-                         GimpPortion       *portion,
+                         GimpArea          *area,
                          Validate           how);
   gboolean  (* use)     (GimpBuffer        *buffer,
                          GimpPortion       *portion, 
@@ -180,54 +255,54 @@ struct _GimpBufferClass
 };
 
 
-
-
-#define gpa_tag(gpa)         ((gpa)->tag)
-#define gpa_width(gpa)       ((gpa)->width)
-#define gpa_height(gpa)      ((gpa)->height)
-#define gpa_pixstride(gpa)   ((gpa)->pixstride)
-#define gpa_rowstride(gpa)   ((gpa)->rowstride)
-#define gpa_data(gpa)        ((gpa)->data)
-#define gpa_offset(gpa)      ((gpa)->offset)
-
-#define gpa_dataptr(gpa)     ((void*) \
-                              (guchar_d((gpa)->data) + \
-                               (gpa)->offset))
-
-
 gint        gimp_buffer_width    (GimpBuffer        *buffer);
 gint        gimp_buffer_height   (GimpBuffer        *buffer);
 gint        gimp_buffer_depth    (GimpBuffer        *buffer);
+
+gint        gimp_buffer_tilenum  (GimpBuffer        *buffer,
+                                  GimpPoint         *point);
+
 void        gimp_buffer_delete   (GimpBuffer        *buffer);
+
 gboolean    gimp_buffer_focus    (GimpBuffer        *buffer,
                                   GimpPortion       *portion,
                                   GimpArea          *roi);
+
 gboolean    gimp_buffer_alloc    (GimpBuffer        *buffer,
-                                  GimpPortion       *portion,
+                                  GimpArea          *roi,
                                   Alloc              how);
+
+gboolean    gimp_buffer_map      (GimpBuffer        *dst,
+                                  GimpArea          *darea,
+                                  GimpBuffer        *src,
+                                  GimpArea          *sarea);
+
 gboolean    gimp_buffer_validate (GimpBuffer        *buffer,
-                                  GimpPortion       *portion,
+                                  GimpArea          *area,
                                   Validate           how);
+
 gboolean    gimp_buffer_use      (GimpBuffer        *buffer,
                                   GimpPortion       *portion,
                                   Use                how);
+
 gboolean    gimp_buffer_query    (GimpBuffer        *buffer,
                                   GimpPortion       *portion,
                                   GimpMemStatus     *status);
+
 gboolean    gimp_buffer_data     (GimpBuffer        *buffer,
                                   GimpPortion       *portion,
                                   GimpPixelArray    *array);
 
-
 /* for use by subclasses */
 void        gimp_buffer_init     (GimpBuffer *buffer,
                                   Tag         tag,
                                   gint        width,
                                   gint        height,
                                   gint        x_period,
-                                  gint        x_phase,
+                                  gint        x_offset,
                                   gint        y_period,
-                                  gint        y_phase);
+                                  gint        y_offset);
+
 void        gimp_buffer_uninit   (GimpBuffer *buffer);
 
 
