@@ -22,6 +22,7 @@
 
 #define DATA   (&flat_buffer->data)
 #define VALID  (flat_buffer->valid)
+#define VALID2 (flat_buffer->validating)
 
 
 
@@ -32,18 +33,18 @@ gimp_flat_buffer_delete (GimpBuffer * buffer)
   GimpFlatBuffer * flat_buffer = GIMP_FLAT_BUFFER (buffer);
 
   d_uninit (DATA);
-  gimp_buffer_uninit (GIMP_BUFFER (flat_buffer));
+  gimp_buffer_uninit (buffer);
   g_free (flat_buffer);
 }
 
 
 static gboolean
 gimp_flat_buffer_alloc (GimpBuffer   *buffer,
-                        GimpPortion  *portion,
+                        GimpArea     *area,
                         Alloc         how)
 {
   GimpFlatBuffer * flat_buffer = GIMP_FLAT_BUFFER (buffer);
-  gint n = buffer->width * buffer->height * buffer->depth;
+  gint n = buffer->_x.len * buffer->_y.len * buffer->_z.len;
 
   switch (how)
     {
@@ -67,7 +68,7 @@ gimp_flat_buffer_alloc (GimpBuffer   *buffer,
 
 static gboolean
 gimp_flat_buffer_validate (GimpBuffer  *buffer,
-                           GimpPortion *portion, 
+                           GimpArea    *area, 
                            Validate     how)
 {
   GimpFlatBuffer * flat_buffer = GIMP_FLAT_BUFFER (buffer);
@@ -80,10 +81,7 @@ gimp_flat_buffer_validate (GimpBuffer  *buffer,
 
       if (! d_is_alloced (DATA))
         {
-          if (buffer->autoalloc == FALSE)
-            return FALSE;
-          
-          if (gimp_flat_buffer_alloc (buffer, portion,
+          if (gimp_flat_buffer_alloc (buffer, area,
                                       ALLOC_ALLOC) != TRUE)
             {
               g_warning ("flatbuffer autoalloc failed");
@@ -91,11 +89,26 @@ gimp_flat_buffer_validate (GimpBuffer  *buffer,
             }
         }
 
-      /* HOOK: PERFORM VALIDATION HERE */
       g_return_val_if_fail (d_write (DATA) == TRUE, FALSE);
-      memset (DATA->data, 0, (buffer->width *
-                              buffer->height *
-                              buffer->depth));
+      if (buffer->vfunc != NULL)
+        {
+          if (VALID2 == TRUE)
+            {
+              g_warning ("recursive validate");
+            }
+          else
+            {
+              VALID2 = TRUE;
+              buffer->vfunc (buffer, area, guchar_d (DATA));
+              VALID2 = FALSE;
+            }
+        }
+      else
+        {
+          memset (guchar_d (DATA), 128, (buffer->_x.len *
+                                         buffer->_y.len *
+                                         buffer->_z.len));
+        }
       g_return_val_if_fail (d_release (DATA) == TRUE, FALSE);
 
       VALID = TRUE;
@@ -129,10 +142,7 @@ gimp_flat_buffer_use (GimpBuffer  *buffer,
                           (how == USE_UPDATE) ||
                           (how == USE_WRITE)))
     {
-      if (buffer->autovalidate == FALSE)
-        return FALSE;
-          
-      if (gimp_flat_buffer_validate (buffer, portion,
+      if (gimp_flat_buffer_validate (buffer, &portion->bounds,
                                      VALIDATE_VALIDATE) != TRUE)
         {
           g_warning ("flatbuffer autovalidate failed");
@@ -190,13 +200,13 @@ gimp_flat_buffer_data (GimpBuffer      *buffer,
   GimpFlatBuffer * flat_buffer = GIMP_FLAT_BUFFER (buffer);
 
   /* describe the memory layout */
-  array->tag       = buffer->tag;
-  array->width     = MIN (buffer->width,
+  array->tag       = buffer->_z.tag;
+  array->width     = MIN (buffer->_x.len,
                           portion->focus.b.x) - portion->focus.a.x;
-  array->height    = MIN (buffer->height,
+  array->height    = MIN (buffer->_y.len,
                           portion->focus.b.y) - portion->focus.a.y;
-  array->pixstride = buffer->depth;
-  array->rowstride = buffer->depth * buffer->width;
+  array->pixstride = buffer->_z.len;
+  array->rowstride = buffer->_z.len * buffer->_x.len;
 
   /* get a pointer to the memory */
   array->data = DATA;
@@ -214,8 +224,8 @@ static GimpFlatBufferClass my_class =
   {
     BUFFER_FLAT,
     gimp_flat_buffer_delete,
-    NULL, /* focus */
     gimp_flat_buffer_alloc,
+    NULL, /* map */
     gimp_flat_buffer_validate,
     gimp_flat_buffer_use,
     gimp_flat_buffer_query,
@@ -237,12 +247,13 @@ gimp_flat_buffer_new (Tag   tag,
 
   flat_buffer = g_new (GimpFlatBuffer, 1);
 
-  gimp_buffer_init (GIMP_BUFFER (flat_buffer),
-                    tag, width, height,
-                    width, 0, height, 0);
+  gimp_buffer_init (GIMP_BUFFER (flat_buffer), tag,
+                    width, width, 0,
+                    height, height, 0);
 
   d_init (DATA);
   VALID = FALSE;
+  VALID2 = FALSE;
 
   GIMP_BUFFER (flat_buffer)->klass = (void*) &my_class;
 
