@@ -18,7 +18,7 @@
 #ifndef __DEBUG_H__
 #define __DEBUG_H__
 
-#include "memory.h"
+#include "cow.h"
 
 void DEBUG_mem    (GimpMem *);
 void DEBUG_memptr (GimpMemPtr *);
